#include "vtkMPIController.h"
#include "vtkSmartPointer.h"
#include "vtkRTAnalyticSource.h"
#include "vtkContourFilter.h"
#include "vtkCompositeRenderManager.h"
#include "vtkRenderer.h"
#include "vtkRenderWindow.h"
#include "vtkRenderWindowInteractor.h"
#include "vtkPolyDataMapper.h"
#include "vtkActor.h"

int main(int argc, char** argv)
{
  vtkSmartPointer<vtkMPIController> controller =
    vtkSmartPointer<vtkMPIController>::New();
  controller->Initialize(&argc, &argv);

  int myid = controller->GetLocalProcessId();

  // TODO: Create a vtkRTAnalyticSource
  vtkSmartPointer<vtkRTAnalyticSource> source =
    vtkSmartPointer<vtkRTAnalyticSource>::New();
  
  // TODO: Create a vtkContourFilter
  vtkSmartPointer<vtkContourFilter> contour =
    vtkSmartPointer<vtkContourFilter>::New();
  
  // TODO: Connect the contour filter to the vtkRTAnalyticSource
  contour->SetInputConnection(source->GetOutputPort());

  // TODO: Set the contour value to 157
  contour->SetNumberOfContours(1);
  contour->SetValue(0, 157);
  
  // create the render window, renderer, and interactor
  vtkSmartPointer<vtkRenderer> aren = vtkSmartPointer<vtkRenderer>::New();
  vtkSmartPointer<vtkRenderWindow> renWin  = 
    vtkSmartPointer<vtkRenderWindow>::New();
  renWin->AddRenderer(aren);
  
  vtkSmartPointer<vtkRenderWindowInteractor> iren = 
    vtkSmartPointer<vtkRenderWindowInteractor>::New();
  iren->SetRenderWindow(renWin);
  
  // associate the contour with a mapper and the mapper to an actor
  vtkSmartPointer<vtkPolyDataMapper> mapper = 
    vtkSmartPointer<vtkPolyDataMapper>::New();
  
  mapper->SetInputConnection(contour->GetOutputPort());
  
  vtkSmartPointer<vtkActor> actor = vtkSmartPointer<vtkActor>::New();
  actor->SetMapper(mapper);
  
  // add the actor to the renderer and start handling events
  aren->AddActor(actor);
  
  // TODO: Create a vtkCompositeRenderManager
  vtkSmartPointer<vtkCompositeRenderManager> tc = 
    vtkSmartPointer<vtkCompositeRenderManager>::New();

  // TODO: Set the render window: SetRenderWindow
  tc->SetRenderWindow(renWin);

  // TODO: Set the MPI controller
  tc->SetController(controller);

  // TODO: Call the InitializePieces() function
  tc->InitializePieces();

  if (myid == 0)
    {
    tc->ResetCamera(aren);
    }

  // TODO: Start the interaction with StartInteractor()
  tc->StartInteractor();

  controller->Finalize();
  return 0;
}
